@@ -1069,6 +1069,11 @@ sub SendEmail {
         }
     }
 
+    my $From = $Kernel::OM->Get('Kernel::System::TemplateGenerator')->Sender(
+        QueueID => $Ticket{QueueID},
+        UserID  => $Self->{UserID},
+    );
+
     # check if there is an error
     if (%Error) {
 
@@ -1173,7 +1178,7 @@ sub SendEmail {
         TicketID       => $Self->{TicketID},
         HistoryType    => 'EmailAgent',
         HistoryComment => "\%\%$To",
-        From           => $GetParam{From},
+        From           => $From,
         To             => $GetParam{To},
         Cc             => $GetParam{Cc},
         Bcc            => $GetParam{Bcc},
@@ -1930,7 +1935,7 @@ sub _GetExtendedParams {
     # get params
     my %GetParam;
     for my $Key (
-        qw(From To Cc Bcc Subject Body ComposeStateID ArticleTypeID
+        qw(To Cc Bcc Subject Body ComposeStateID ArticleTypeID
         ArticleID TimeUnits Year Month Day Hour Minute FormID)
         )
     {
