@@ -31,7 +31,7 @@ sub new {
 
     # get params
     for (
-        qw(From To Cc Bcc Subject Body InReplyTo References ComposeStateID ArticleTypeID
+        qw(To Cc Bcc Subject Body InReplyTo References ComposeStateID ArticleTypeID
         ArticleID TimeUnits Year Month Day Hour Minute FormID)
         )
     {
@@ -925,6 +925,11 @@ sub SendEmail {
         FormID => $GetParam{FormID},
     );
 
+    my $From = $Kernel::OM->Get('Kernel::System::TemplateGenerator')->Sender(
+        QueueID => $Ticket{QueueID},
+        UserID  => $Self->{UserID},
+    );
+
     # check if there is an error
     if (%Error) {
 
@@ -1043,7 +1048,7 @@ sub SendEmail {
         TicketID       => $Self->{TicketID},
         HistoryType    => 'Forward',
         HistoryComment => "\%\%$To",
-        From           => $GetParam{From},
+        From           => $From,
         To             => $GetParam{To},
         Cc             => $GetParam{Cc},
         Bcc            => $GetParam{Bcc},
