@@ -1,14 +1,14 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, http://otrs.com/
+# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
-# the enclosed file COPYING for license information (AGPL). If you
-# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
+# the enclosed file COPYING for license information (GPL). If you
+# did not receive this file, see https://www.gnu.org/licenses/gpl-3.0.txt.
 # --
 
 package Kernel::Output::Template::Provider;
 ## no critic(Perl::Critic::Policy::OTRS::RequireCamelCase)
-## nofilter(TidyAll::Plugin::OTRS::Perl::SyntaxCheck) # bails on TT Constants
+## nofilter(TidyAll::Plugin::OTRS::Perl::SyntaxCheck)
 
 use strict;
 use warnings;
@@ -110,8 +110,9 @@ sub _fetch {
     # Check if the template exists, is cacheable and if a cached version exists.
     if ( -e $name && $self->{CachingEnabled} ) {
 
+        my $UserTheme      = $self->{LayoutObject}->{EnvRef}->{UserTheme};
         my $template_mtime = $self->_template_modified($name);
-        my $CacheKey       = $self->_compiled_filename($name) . '::' . $template_mtime;
+        my $CacheKey       = $self->_compiled_filename($name) . '::' . $template_mtime . '::' . $UserTheme;
 
         # Is there an up-to-date compiled version in the cache?
         my $Cache = $Kernel::OM->Get('Kernel::System::Cache')->Get(
@@ -242,7 +243,8 @@ sub _compile {
 
         # write the Perl code to the file $compfile, if defined
         if ($compfile) {
-            my $CacheKey = $compfile . '::' . $data->{time};
+            my $UserTheme = $self->{LayoutObject}->{EnvRef}->{UserTheme};
+            my $CacheKey  = $compfile . '::' . $data->{time} . '::' . $UserTheme;
 
             if ( $self->{CachingEnabled} ) {
                 $Kernel::OM->Get('Kernel::System::Cache')->Set(
@@ -274,7 +276,7 @@ sub _compile {
     # return STATUS_ERROR, or STATUS_DECLINED if we're being tolerant
     return $self->{TOLERANT}
         ? ( undef, Template::Constants::STATUS_DECLINED )
-        : ( $error, Template::Constants::STATUS_ERROR )
+        : ( $error, Template::Constants::STATUS_ERROR );
 }
 
 =item store()
@@ -339,7 +341,7 @@ sub _PreProcessTemplateContent {
     #
     # Remove DTL-style comments (lines starting with #)
     #
-    $Content =~ s/^#.*\n//gm;
+    $Content =~ s/^#.*\n//gm if !$ENV{TEMPLATE_KEEP_COMMENTS};
 
     #
     # Insert a BLOCK call into the template.
@@ -400,14 +402,14 @@ sub MigrateDTLtoTT {
     };
 
     # $Quote $Config
-    $Content =~ s{\$Quote{"\$Config{"($ID)"}"}}{[% Config("$1") | html %]}smxg;
+    $Content =~ s{\$Quote[{]"\$Config[{]"($ID)"}"}}{[% Config("$1") | html %]}smxg;
 
     # $Quote $Env
-    $Content =~ s{\$Quote{"\$Env{"($ID)"}"}}{[% Env("$1") | html %]}smxg;
+    $Content =~ s{\$Quote[{]"\$Env[{]"($ID)"}"}}{[% Env("$1") | html %]}smxg;
 
     # $Quote $Data
     $Content =~ s{
-            \$Quote{"\$Data{"($ID)"}"}
+            \$Quote[{]"\$Data[{]"($ID)"}"}
         }
         {
             '[% Data.' . $SafeArrrayAccess->($1) . ' | html %]'
@@ -415,7 +417,7 @@ sub MigrateDTLtoTT {
 
     # $Quote with length
     $Content =~ s{
-            \$Quote{"\$Data{"($ID)"}",\s*"(\d+)"}
+            \$Quote[{]"\$Data[{]"($ID)"}",\s*"(\d+)"}
         }
         {
             '[% Data.' . $SafeArrrayAccess->($1) . " | truncate($2) | html %]"
@@ -423,7 +425,7 @@ sub MigrateDTLtoTT {
 
     # $Quote with dynamic length
     $Content =~ s{
-            \$Quote{"\$Data{"($ID)"}",\s*"\$Q?Data{"($ID)"}"}
+            \$Quote[{]"\$Data[{]"($ID)"}",\s*"\$Q?Data[{]"($ID)"}"}
         }
         {
             '[% Data.' . $SafeArrrayAccess->($1) . ' | truncate(Data.' . $SafeArrrayAccess->($2) . ') | html %]'
@@ -431,7 +433,7 @@ sub MigrateDTLtoTT {
 
     # $Quote with translated text and fixed length
     $Content =~ s{
-            \$Quote{"\$Text{"\$Data{"($ID)"}"}",\s*"(\d+)"}
+            \$Quote[{]"\$Text[{]"\$Data[{]"($ID)"}"}",\s*"(\d+)"}
         }
         {
             '[% Data.' . $SafeArrrayAccess->($1) . " | Translate | truncate($2) | html %]"
@@ -439,7 +441,7 @@ sub MigrateDTLtoTT {
 
     # $Quote with translated text and dynamic length
     $Content =~ s{
-            \$Quote{"\$Text{"\$Data{"($ID)"}"}",\s*"\$Q?Data{"($ID)"}"}
+            \$Quote[{]"\$Text[{]"\$Data[{]"($ID)"}"}",\s*"\$Q?Data[{]"($ID)"}"}
         }
         {
             '[% Data.' . $SafeArrrayAccess->($1) . ' | Translate | truncate(Data.' . $SafeArrrayAccess->($2) . ') | html %]'
@@ -455,22 +457,22 @@ sub MigrateDTLtoTT {
         my $Result = '[% ';
 
         # Text contains a tag
-        if ( $Text =~ m{\$TimeLong{"\$Q?Data{"($ID)"}"}}smx ) {
+        if ( $Text =~ m{\$TimeLong[{]"\$Q?Data[{]"($ID)"}"}}smx ) {
             $Result .= "Translate(Localize(Data." . $SafeArrrayAccess->($1) . ", \"TimeLong\")";
         }
-        elsif ( $Text =~ m{\$TimeShort{"\$Q?Data{"($ID)"}"}}smx ) {
+        elsif ( $Text =~ m{\$TimeShort[{]"\$Q?Data[{]"($ID)"}"}}smx ) {
             $Result .= "Translate(Localize(Data." . $SafeArrrayAccess->($1) . ", \"TimeShort\")";
         }
-        elsif ( $Text =~ m{\$Date{"\$Q?Data{"($ID)"}"}}smx ) {
+        elsif ( $Text =~ m{\$Date[{]"\$Q?Data[{]"($ID)"}"}}smx ) {
             $Result .= "Translate(Localize(Data." . $SafeArrrayAccess->($1) . ", \"Date\")";
         }
-        elsif ( $Text =~ m{\$Q?Data{"($ID)"}}smx ) {
+        elsif ( $Text =~ m{\$Q?Data[{]"($ID)"}}smx ) {
             $Result .= "Translate(Data." . $SafeArrrayAccess->($1) . "";
         }
-        elsif ( $Text =~ m{\$Config{"($ID)"}}smx ) {
+        elsif ( $Text =~ m{\$Config[{]"($ID)"}}smx ) {
             $Result .= "Translate(Config(\"$1\")";
         }
-        elsif ( $Text =~ m{\$Q?Env{"($ID)"}}smx ) {
+        elsif ( $Text =~ m{\$Q?Env[{]"($ID)"}}smx ) {
             $Result .= "Translate(Env(\"$1\")";
         }
 
@@ -488,22 +490,22 @@ sub MigrateDTLtoTT {
         PARAMETER:
         for my $Parameter (@Parameters) {
             next PARAMETER if ( !$Parameter );
-            if ( $Parameter =~ m{\$TimeLong{"\$Q?Data{"($ID)"}"}}smx ) {
+            if ( $Parameter =~ m{\$TimeLong[{]"\$Q?Data[{]"($ID)"}"}}smx ) {
                 $Result .= ", Localize(Data.$1, \"TimeLong\")";
             }
-            elsif ( $Parameter =~ m{\$TimeShort{"\$Q?Data{"($ID)"}"}}smx ) {
+            elsif ( $Parameter =~ m{\$TimeShort[{]"\$Q?Data[{]"($ID)"}"}}smx ) {
                 $Result .= ", Localize(Data.$1, \"TimeShort\")";
             }
-            elsif ( $Parameter =~ m{\$Date{"\$Q?Data{"($ID)"}"}}smx ) {
+            elsif ( $Parameter =~ m{\$Date[{]"\$Q?Data[{]"($ID)"}"}}smx ) {
                 $Result .= ", Localize(Data.$1, \"Date\")";
             }
-            elsif ( $Parameter =~ m{\$Q?Data{"($ID)"}}smx ) {
+            elsif ( $Parameter =~ m{\$Q?Data[{]"($ID)"}}smx ) {
                 $Result .= ", Data.$1";
             }
-            elsif ( $Parameter =~ m{\$Config{"($ID)"}}smx ) {
+            elsif ( $Parameter =~ m{\$Config[{]"($ID)"}}smx ) {
                 $Result .= ", Config(\"$1\")";
             }
-            elsif ( $Parameter =~ m{\$Q?Env{"($ID)"}}smx ) {
+            elsif ( $Parameter =~ m{\$Q?Env[{]"($ID)"}}smx ) {
                 $Result .= ", Env(\"$1\")";
             }
             else {
@@ -527,28 +529,28 @@ sub MigrateDTLtoTT {
 
     # $Text
     $Content =~ s{
-            \$Text{
+            \$Text[{]
                 ["']
                 (
                     [^\$]+?
-                    |\$Q?Data{\"$ID\"}
-                    |\$Config{\"$ID\"}
-                    |\$Q?Env{\"$ID\"}
-                    |\$TimeLong{\"\$Q?Data{\"$ID\"}\"}
-                    |\$TimeShort{\"\$Q?Data{\"$ID\"}\"}
-                    |\$Date{\"\$Q?Data{\"$ID\"}\"}
+                    |\$Q?Data[{]\"$ID\"}
+                    |\$Config[{]\"$ID\"}
+                    |\$Q?Env[{]\"$ID\"}
+                    |\$TimeLong[{]\"\$Q?Data[{]\"$ID\"}\"}
+                    |\$TimeShort[{]\"\$Q?Data[{]\"$ID\"}\"}
+                    |\$Date[{]\"\$Q?Data[{]\"$ID\"}\"}
                 )
                 ["']
                 ((?:
                     ,\s*["']
                     (?:
                         [^\$]+?
-                        |\$Q?Data{\"$ID\"}
-                        |\$Config{\"$ID\"}
-                        |\$Q?Env{\"$ID\"}
-                        |\$TimeLong{\"\$Q?Data{\"$ID\"}\"}
-                        |\$TimeShort{\"\$Q?Data{\"$ID\"}\"}
-                        |\$Date{\"\$Q?Data{\"$ID\"}\"}
+                        |\$Q?Data[{]\"$ID\"}
+                        |\$Config[{]\"$ID\"}
+                        |\$Q?Env[{]\"$ID\"}
+                        |\$TimeLong[{]\"\$Q?Data[{]\"$ID\"}\"}
+                        |\$TimeShort[{]\"\$Q?Data[{]\"$ID\"}\"}
+                        |\$Date[{]\"\$Q?Data[{]\"$ID\"}\"}
                     )
                 ["'])*)
             }
@@ -562,28 +564,28 @@ sub MigrateDTLtoTT {
 
     # $JSText
     $Content =~ s{
-            ["']\$JSText{
+            ["']\$JSText[{]
                 ["']
                 (
                     [^\$]+?
-                    |\$Q?Data{\"$ID\"}
-                    |\$Config{\"$ID\"}
-                    |\$Q?Env{\"$ID\"}
-                    |\$TimeLong{\"\$Q?Data{\"$ID\"}\"}
-                    |\$TimeShort{\"\$Q?Data{\"$ID\"}\"}
-                    |\$Date{\"\$Q?Data{\"$ID\"}\"}
+                    |\$Q?Data[{]\"$ID\"}
+                    |\$Config[{]\"$ID\"}
+                    |\$Q?Env[{]\"$ID\"}
+                    |\$TimeLong[{]\"\$Q?Data[{]\"$ID\"}\"}
+                    |\$TimeShort[{]\"\$Q?Data[{]\"$ID\"}\"}
+                    |\$Date[{]\"\$Q?Data[{]\"$ID\"}\"}
                 )
                 ["']
                 ((?:
                     ,\s*["']
                     (?:
                         [^\$]+?
-                        |\$Q?Data{\"$ID\"}
-                        |\$Config{\"$ID\"}
-                        |\$Q?Env{\"$ID\"}
-                        |\$TimeLong{\"\$Q?Data{\"$ID\"}\"}
-                        |\$TimeShort{\"\$Q?Data{\"$ID\"}\"}
-                        |\$Date{\"\$Q?Data{\"$ID\"}\"}
+                        |\$Q?Data[{]\"$ID\"}
+                        |\$Config[{]\"$ID\"}
+                        |\$Q?Env[{]\"$ID\"}
+                        |\$TimeLong[{]\"\$Q?Data[{]\"$ID\"}\"}
+                        |\$TimeShort[{]\"\$Q?Data[{]\"$ID\"}\"}
+                        |\$Date[{]\"\$Q?Data[{]\"$ID\"}\"}
                     )
                 ["'])*)
             }
@@ -594,17 +596,17 @@ sub MigrateDTLtoTT {
         }esmxg;
 
     # $TimeLong
-    $Content =~ s{\$TimeLong{"\$Q?Data{"($ID)"}"}}{[% Data.$1 | Localize("TimeLong") %]}smxg;
+    $Content =~ s{\$TimeLong[{]"\$Q?Data[{]"($ID)"}"}}{[% Data.$1 | Localize("TimeLong") %]}smxg;
 
     # $TimeShort
-    $Content =~ s{\$TimeShort{"\$Q?Data{"($ID)"}"}}{[% Data.$1 | Localize("TimeShort") %]}smxg;
+    $Content =~ s{\$TimeShort[{]"\$Q?Data[{]"($ID)"}"}}{[% Data.$1 | Localize("TimeShort") %]}smxg;
 
     # $Date
-    $Content =~ s{\$Date{"\$Q?Data{"($ID)"}"}}{[% Data.$1 | Localize("Date") %]}smxg;
+    $Content =~ s{\$Date[{]"\$Q?Data[{]"($ID)"}"}}{[% Data.$1 | Localize("Date") %]}smxg;
 
     # $QData with length
     $Content =~ s{
-            \$QData{"($ID)",\s*"(\d+)"}
+            \$QData[{]"($ID)",\s*"(\d+)"}
         }
         {
             "[% Data." . $SafeArrrayAccess->($1) . " | truncate($2) | html %]"
@@ -612,7 +614,7 @@ sub MigrateDTLtoTT {
 
     # simple $QData
     $Content =~ s{
-            \$QData{"($ID)"}
+            \$QData[{]"($ID)"}
         }
         {
             "[% Data." . $SafeArrrayAccess->($1) . " | html %]"
@@ -620,7 +622,7 @@ sub MigrateDTLtoTT {
 
     # $LQData
     $Content =~ s{
-            \$LQData{"($ID)"}
+            \$LQData[{]"($ID)"}
         }
         {
             "[% Data." . $SafeArrrayAccess->($1) . " | uri %]"
@@ -628,20 +630,20 @@ sub MigrateDTLtoTT {
 
     # simple $Data
     $Content =~ s{
-            \$Data{"($ID)"}
+            \$Data[{]"($ID)"}
         }
         {
             "[% Data." . $SafeArrrayAccess->($1) . " %]"
         }esmxg;
 
     # $Config
-    $Content =~ s{\$Config{"($ID)"}}{[% Config("$1") %]}smxg;
+    $Content =~ s{\$Config[{]"($ID)"}}{[% Config("$1") %]}smxg;
 
     # $Env
-    $Content =~ s{\$Env{"($ID)"}}{[% Env("$1") %]}smxg;
+    $Content =~ s{\$Env[{]"($ID)"}}{[% Env("$1") %]}smxg;
 
     # $QEnv
-    $Content =~ s{\$QEnv{"($ID)"}}{[% Env("$1") | html %]}smxg;
+    $Content =~ s{\$QEnv[{]"($ID)"}}{[% Env("$1") | html %]}smxg;
 
     # dtl:block
     my %BlockSeen;
@@ -667,7 +669,7 @@ sub MigrateDTLtoTT {
         =~ s{<!--\s*dtl:js_on_document_complete_placeholder\s*-->}{[% PROCESS JSOnDocumentCompleteInsert %]}smxg;
 
     # $Include
-    $Content =~ s{\$Include{"($ID)"}}{[% InsertTemplate("$1.tt") %]}smxg;
+    $Content =~ s{\$Include[{]"($ID)"}}{[% InsertTemplate("$1.tt") %]}smxg;
 
     my ( $Counter, $ErrorMessage );
     LINE:
@@ -691,10 +693,10 @@ sub MigrateDTLtoTT {
 
 =head1 TERMS AND CONDITIONS
 
-This software is part of the OTRS project (L<http://otrs.org/>).
+This software is part of the OTRS project (L<https://otrs.org/>).
 
 This software comes with ABSOLUTELY NO WARRANTY. For details, see
-the enclosed file COPYING for license information (AGPL). If you
-did not receive this file, see L<http://www.gnu.org/licenses/agpl.txt>.
+the enclosed file COPYING for license information (GPL). If you
+did not receive this file, see L<https://www.gnu.org/licenses/gpl-3.0.txt>.
 
 =cut
