@@ -1,5 +1,5 @@
 # --
-# Copyright (C) 2001-2018 OTRS AG, https://otrs.com/
+# Copyright (C) 2001-2019 OTRS AG, https://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -11,10 +11,8 @@ package Kernel::System::VariableCheck;
 use strict;
 use warnings;
 
-use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
-
-use Exporter;
-%EXPORT_TAGS = (    ## no critic
+use Exporter qw(import);
+our %EXPORT_TAGS = (    ## no critic
     all => [
         'IsArrayRefWithData',
         'IsHashRefWithData',
@@ -32,8 +30,6 @@ use Exporter;
 );
 Exporter::export_ok_tags('all');
 
-@ISA = qw(Exporter);
-
 =head1 NAME
 
 Kernel::System::VariableCheck - helper functions to check variables
@@ -376,7 +372,7 @@ sub IsIPv6Address {
 
 =item IsMD5Sum()
 
-test supplied data to determine if it is an md5sum (32 hex characters)
+test supplied data to determine if it is an C<MD5> sum (32 hex characters)
 
 returns 1 if data matches criteria or undef otherwise
 
@@ -413,11 +409,6 @@ ARRAY or HASH.
 sub DataIsDifferent {
     my (%Param) = @_;
 
-    # check needed stuff
-    for (qw(Data1 Data2)) {
-        return if ( !defined $Param{$_} );
-    }
-
     # ''
     if ( ref $Param{Data1} eq '' && ref $Param{Data2} eq '' ) {
 
